==> use-switch-tabs-0.1.1.tgz: package/es/useSwitchTabs.js <==
var _excluded = ["title", "location", "content"];

function _objectWithoutProperties(source, excluded) { if (source == null) return {}; var target = _objectWithoutPropertiesLoose(source, excluded); var key, i; if (Object.getOwnPropertySymbols) { var sourceSymbolKeys = Object.getOwnPropertySymbols(source); for (i = 0; i < sourceSymbolKeys.length; i++) { key = sourceSymbolKeys[i]; if (excluded.indexOf(key) >= 0) continue; if (!Object.prototype.propertyIsEnumerable.call(source, key)) continue; target[key] = source[key]; } } return target; }

function _objectWithoutPropertiesLoose(source, excluded) { if (source == null) return {}; var target = {}; var sourceKeys = Object.keys(source); var key, i; for (i = 0; i < sourceKeys.length; i++) { key = sourceKeys[i]; if (excluded.indexOf(key) >= 0) continue; target[key] = source[key]; } return target; }

function _toConsumableArray(arr) { return _arrayWithoutHoles(arr) || _iterableToArray(arr) || _unsupportedIterableToArray(arr) || _nonIterableSpread(); }

function _nonIterableSpread() { throw new TypeError("Invalid attempt to spread non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method."); }

function _iterableToArray(iter) { if (typeof Symbol !== "undefined" && iter[Symbol.iterator] != null || iter["@@iterator"] != null) return Array.from(iter); }

function _arrayWithoutHoles(arr) { if (Array.isArray(arr)) return _arrayLikeToArray(arr); }

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); if (enumerableOnly) { symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; }); } keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; if (i % 2) { ownKeys(Object(source), true).forEach(function (key) { _defineProperty(target, key, source[key]); }); } else if (Object.getOwnPropertyDescriptors) { Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)); } else { ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

function _slicedToArray(arr, i) { return _arrayWithHoles(arr) || _iterableToArrayLimit(arr, i) || _unsupportedIterableToArray(arr, i) || _nonIterableRest(); }

function _nonIterableRest() { throw new TypeError("Invalid attempt to destructure non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method."); }

function _unsupportedIterableToArray(o, minLen) { if (!o) return; if (typeof o === "string") return _arrayLikeToArray(o, minLen); var n = Object.prototype.toString.call(o).slice(8, -1); if (n === "Object" && o.constructor) n = o.constructor.name; if (n === "Map" || n === "Set") return Array.from(o); if (n === "Arguments" || /^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n)) return _arrayLikeToArray(o, minLen); }

function _arrayLikeToArray(arr, len) { if (len == null || len > arr.length) len = arr.length; for (var i = 0, arr2 = new Array(len); i < len; i++) { arr2[i] = arr[i]; } return arr2; }

function _iterableToArrayLimit(arr, i) { var _i = arr == null ? null : typeof Symbol !== "undefined" && arr[Symbol.iterator] || arr["@@iterator"]; if (_i == null) return; var _arr = []; var _n = true; var _d = false; var _s, _e; try { for (_i = _i.call(arr); !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i["return"] != null) _i["return"](); } finally { if (_d) throw _e; } } return _arr; }

function _arrayWithHoles(arr) { if (Array.isArray(arr)) return arr; }

import React, { useEffect, useState, useMemo, useRef } from 'react';
import { useLocalStorageState, usePersistFn, usePrevious } from 'ahooks';
import _find from 'lodash/find';
import _findIndex from 'lodash/findIndex';
import _isEqual from 'lodash/isEqual';
import _omit from 'lodash/omit';
import _isArray from 'lodash/isArray';
import { getRenderRoute, getRenderRouteKey } from './utils';
import { Mode } from './config';

function useSwitchTabs(options) {
  var _options$mode = options.mode,
      mode = _options$mode === void 0 ? Mode.Route : _options$mode,
      setTabName = options.setTabName,
      originalRoutes = options.originalRoutes,
      persistent = options.persistent,
      location = options.location,
      history = options.history,
      children = options.children,
      propsActionRef = options.actionRef;

  var currentTabLocation = _omit(location, ['key']);

  var actionRef = useRef();

  var _useLocalStorageState = useLocalStorageState('tabLocations', []),
      _useLocalStorageState2 = _slicedToArray(_useLocalStorageState, 2),
      tabLocations = _useLocalStorageState2[0],
      setTabLocations = _useLocalStorageState2[1];

  var _useState = useState(function () {
    if (persistent && _isArray(tabLocations) && tabLocations.length) {
      return tabLocations.map(function (tabLocation) {
        var renderRoute = getRenderRoute({
          location: tabLocation,
          mode: mode,
          originalRoutes: originalRoutes,
          setTabName: setTabName
        });
        return {
          title: renderRoute.name,
          key: getRenderRouteKey(renderRoute, mode),
          content: /*#__PURE__*/React.cloneElement(children, {
            location: tabLocation
          }),
          location: tabLocation
        };
      });
    }

    return [];
  }),
      _useState2 = _slicedToArray(_useState, 2),
      tabs = _useState2[0],
      setTabs = _useState2[1];

  var currentRenderRoute = getRenderRoute({
    location: currentTabLocation,
    mode: mode,
    originalRoutes: originalRoutes,
    setTabName: setTabName
  });
  var currentTabKey = useMemo(function () {
    return getRenderRouteKey(currentRenderRoute, mode);
  }, [mode, currentRenderRoute]);
  var prevActiveKey = usePrevious(currentTabKey, function (prev, next) {
    return prev !== next;
  });
  var getTab = usePersistFn(function (tabKey) {
    return _find(tabs, {
      key: tabKey
    });
  });
  var processTabs = usePersistFn(function (_tabs) {
    return _tabs.map(function (item) {
      return _tabs.length === 1 ? _objectSpread(_objectSpread({}, item), {}, {
        closable: false
      }) : item;
    });
  });
  /** 获取激活标签页的相邻标签页 */

  var getNextTab = usePersistFn(function () {
    var removeIndex = _findIndex(tabs, {
      key: currentTabKey
    });

    var nextIndex = removeIndex >= 1 ? removeIndex - 1 : removeIndex + 1;
    return tabs[nextIndex];
  });
  /**
   * force: 是否在目标标签页不存在的时候强制回调函数
   */

  var handleSwitch = usePersistFn(function (keyToSwitch, callback) {
    var force = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : false;

    if (!keyToSwitch) {
      return;
    }
    /**
     * `keyToSwitch` 有值时，`targetTab` 可能为空。
     *
     * 如：一个会调用 `window.closeAndGoBackTab(path)` 的页面在 F5 刷新之后
     */


    var targetTab = getTab(keyToSwitch);
    history.push(targetTab ? targetTab.location : keyToSwitch);

    if (force) {
      callback === null || callback === void 0 ? void 0 : callback();
    } else {
      targetTab && (callback === null || callback === void 0 ? void 0 : callback());
    }
  });
  /** 删除标签页处理事件，可接收一个 `nextTabKey` 参数，自定义需要返回的标签页 */

  var handleRemove = usePersistFn(function (removeKey, nextTabKey, callback, force) {
    if (tabs.length === 1) {
      console.warn('the final tab, can not remove.');
      return;
    }

    var getNextTabKeyByRemove = function getNextTabKeyByRemove() {
      var _getNextTab;

      return removeKey === currentTabKey ? (_getNextTab = getNextTab()) === null || _getNextTab === void 0 ? void 0 : _getNextTab.key : currentTabKey;
    };

    handleSwitch(nextTabKey || getNextTabKeyByRemove(), callback, force);
    setTabs(function (prevTabs) {
      return processTabs(prevTabs.filter(function (item) {
        return item.key !== removeKey;
      }));
    });
  });
  var handleRemoveOthers = usePersistFn(function (currentKey, callback) {
    handleSwitch(currentKey, callback);
    setTabs(function (prevTabs) {
      return processTabs(prevTabs.filter(function (item) {
        return item.key === currentKey;
      }));
    });
  });
  var handRemoveRightTabs = usePersistFn(function (currentKey, callback) {
    handleSwitch(getTab(currentKey).key, callback);
    setTabs(function (prevTabs) {
      return processTabs(prevTabs.slice(0, _findIndex(prevTabs, {
        key: currentKey
      }) + 1));
    });
  });
  /**
   * 新增第一个 tab 不可删除
   *
   * @param newTab
   */

  var addTab = usePersistFn(function (newTab, follow) {
    setTabs(function (prevTabs) {
      var result = _toConsumableArray(prevTabs);

      if (follow) {
        var targetIndex = _findIndex(prevTabs, function (tab) {
          if (mode === Mode.Route) {
            return tab.key === follow;
          }

          var followReg = new RegExp("^".concat(follow));
          return followReg.test(tab.key);
        });

        if (targetIndex >= 0) {
          result.splice(targetIndex + 1, 0, newTab);
        } else {
          result = [].concat(_toConsumableArray(result), [newTab]);
        }
      } else {
        result = [].concat(_toConsumableArray(result), [newTab]);
      }

      return result.map(function (item, index) {
        return tabs.length === 0 && index === 0 ? _objectSpread(_objectSpread({}, item), {}, {
          closable: false
        }) : _objectSpread(_objectSpread({}, item), {}, {
          closable: true
        });
      });
    });
  });
  /**
   * 重载标签页，传入参数重写相关属性
   *
   * @param reloadKey 需要刷新的 tab key
   * @param tabTitle 需要刷新的 tab 标题
   * @param location 需要刷新的 tab location
   * @param content 需要刷新的 tab 渲染的内容
   */

  var reloadTab = usePersistFn(function () {
    var reloadKey = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : currentTabKey;
    var tabTitle = arguments.length > 1 ? arguments[1] : undefined;
    var tabLocation = arguments.length > 2 ? arguments[2] : undefined;
    var content = arguments.length > 3 ? arguments[3] : undefined;

    if (tabs.length < 1) {
      return;
    }

    if (process.env.NODE_ENV === 'development') {
      console.log("reload tab key: ".concat(reloadKey));
    }

    var updatedTabs = tabs.map(function (item) {
      if (item.key === reloadKey) {
        var prevTabTitle = item.title,
            prevLocation = item.location,
            prevContent = item.content,
            rest = _objectWithoutProperties(item, _excluded);

        return _objectSpread(_objectSpread({}, rest), {}, {
          title: tabTitle || prevTabTitle,
          location: tabLocation || prevLocation,
          content: content || /*#__PURE__*/React.cloneElement(item.content, {
            key: new Date().valueOf()
          })
        });
      }

      return item;
    });
    setTabs(updatedTabs);
  });
  var goBackTab = usePersistFn(function (path, callback, force) {
    if (!path && (!prevActiveKey || !getTab(prevActiveKey))) {
      console.warn('go back failed, no previous actived key or previous tab is closed.');
      return;
    }

    handleSwitch(path || prevActiveKey, callback, force);
  });
  /** 关闭后自动切换到附近的标签页，如果是最后一个标签页不可删除 */

  var closeTab = usePersistFn(function (path, callback, force) {
    if (path && !getTab(path)) {
      console.warn('close failed, target tab is closed.');
      return;
    }

    handleRemove(path || currentTabKey, undefined, callback, force);
  });
  /** 关闭当前标签页并返回到上次打开的标签页 */

  var closeAndGoBackTab = usePersistFn(function (path, callback, force) {
    if (!path && (!prevActiveKey || !getTab(prevActiveKey))) {
      console.warn('close and go back failed, no previous actived key or previous tab is closed.');
      return;
    }

    handleRemove(currentTabKey, path || prevActiveKey, callback, force);
  });
  useEffect(function () {
    if (persistent) {
      setTabLocations(tabs.map(function (item) {
        return item.location;
      }));
      return;
    }

    if (tabLocations) {
      setTabLocations();
    }
  }, [persistent, tabs]);
  useEffect(function () {
    actionRef.current = {
      reloadTab: reloadTab,
      goBackTab: goBackTab,
      closeTab: closeTab,
      closeAndGoBackTab: closeAndGoBackTab
    };
    return function () {
      var hint = function hint() {
        console.warn("useSwitchTabs had unmounted.");
      };

      actionRef.current = {
        reloadTab: hint,
        goBackTab: hint,
        closeTab: hint,
        closeAndGoBackTab: hint
      };
    };
  }, []);
  useEffect(function () {
    var activedTab = getTab(currentTabKey);

    if (activedTab) {
      var prevTabLocation = activedTab.location;

      if (!_isEqual(currentTabLocation, prevTabLocation)) {
        reloadTab(currentTabKey, currentRenderRoute.name, currentTabLocation, children);
      } else {
        if (process.env.NODE_ENV === 'development') {
          console.log("no effect of tab key: ".concat(currentTabKey));
        }
      }
    } else {
      var newTab = {
        title: currentRenderRoute.name,
        key: currentTabKey,
        content: children,
        location: currentTabLocation
      };

      var _ref = currentRenderRoute || {},
          follow = _ref.follow;

      if (process.env.NODE_ENV === 'development') {
        console.log("add tab key: ".concat(currentTabKey));
      }

      addTab(newTab, follow);
    } // 不可将当前 location 作为依赖，否则在操作非当前 location 对应的标签页时会有异常
    // 比如在非当前 location 对应的标签页的标签菜单中触发删除其他标签页，会导致本应只有一个标签页时，
    // 但会再次创建一个当前 location 对应的标签页

  }, [children, currentTabKey]);

  if (propsActionRef) {
    if (typeof propsActionRef === 'function') {
      propsActionRef(actionRef.current);
    } else {
      propsActionRef.current = actionRef.current;
    }
  }

  return {
    tabs: tabs,
    activeKey: currentTabKey,
    handleSwitch: handleSwitch,
    handleRemove: handleRemove,
    handleRemoveOthers: handleRemoveOthers,
    handRemoveRightTabs: handRemoveRightTabs
  };
}

export default useSwitchTabs;

==> use-switch-tabs-0.1.4.tgz: package/es/useSwitchTabs.js <==
var _excluded = ["title", "location", "content"];

function _objectWithoutProperties(source, excluded) { if (source == null) return {}; var target = _objectWithoutPropertiesLoose(source, excluded); var key, i; if (Object.getOwnPropertySymbols) { var sourceSymbolKeys = Object.getOwnPropertySymbols(source); for (i = 0; i < sourceSymbolKeys.length; i++) { key = sourceSymbolKeys[i]; if (excluded.indexOf(key) >= 0) continue; if (!Object.prototype.propertyIsEnumerable.call(source, key)) continue; target[key] = source[key]; } } return target; }

function _objectWithoutPropertiesLoose(source, excluded) { if (source == null) return {}; var target = {}; var sourceKeys = Object.keys(source); var key, i; for (i = 0; i < sourceKeys.length; i++) { key = sourceKeys[i]; if (excluded.indexOf(key) >= 0) continue; target[key] = source[key]; } return target; }

function _toConsumableArray(arr) { return _arrayWithoutHoles(arr) || _iterableToArray(arr) || _unsupportedIterableToArray(arr) || _nonIterableSpread(); }

function _nonIterableSpread() { throw new TypeError("Invalid attempt to spread non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method."); }

function _iterableToArray(iter) { if (typeof Symbol !== "undefined" && iter[Symbol.iterator] != null || iter["@@iterator"] != null) return Array.from(iter); }

function _arrayWithoutHoles(arr) { if (Array.isArray(arr)) return _arrayLikeToArray(arr); }

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); if (enumerableOnly) { symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; }); } keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; if (i % 2) { ownKeys(Object(source), true).forEach(function (key) { _defineProperty(target, key, source[key]); }); } else if (Object.getOwnPropertyDescriptors) { Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)); } else { ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

function _slicedToArray(arr, i) { return _arrayWithHoles(arr) || _iterableToArrayLimit(arr, i) || _unsupportedIterableToArray(arr, i) || _nonIterableRest(); }

function _nonIterableRest() { throw new TypeError("Invalid attempt to destructure non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method."); }

function _unsupportedIterableToArray(o, minLen) { if (!o) return; if (typeof o === "string") return _arrayLikeToArray(o, minLen); var n = Object.prototype.toString.call(o).slice(8, -1); if (n === "Object" && o.constructor) n = o.constructor.name; if (n === "Map" || n === "Set") return Array.from(o); if (n === "Arguments" || /^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n)) return _arrayLikeToArray(o, minLen); }

function _arrayLikeToArray(arr, len) { if (len == null || len > arr.length) len = arr.length; for (var i = 0, arr2 = new Array(len); i < len; i++) { arr2[i] = arr[i]; } return arr2; }

function _iterableToArrayLimit(arr, i) { var _i = arr == null ? null : typeof Symbol !== "undefined" && arr[Symbol.iterator] || arr["@@iterator"]; if (_i == null) return; var _arr = []; var _n = true; var _d = false; var _s, _e; try { for (_i = _i.call(arr); !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i["return"] != null) _i["return"](); } finally { if (_d) throw _e; } } return _arr; }

function _arrayWithHoles(arr) { if (Array.isArray(arr)) return arr; }

import React, { useEffect, useState, useMemo, useRef } from 'react';
import { useLocalStorageState, usePersistFn, usePrevious } from 'ahooks';
import _find from 'lodash/find';
import _findIndex from 'lodash/findIndex';
import _isEqual from 'lodash/isEqual';
import _omit from 'lodash/omit';
import _get from 'lodash/get';
import _isArray from 'lodash/isArray';
import { getRenderRoute, getRenderRouteKey } from './utils';
import { Mode } from './config';

function useSwitchTabs(options) {
  var _options$mode = options.mode,
      mode = _options$mode === void 0 ? Mode.Route : _options$mode,
      originalRoutes = options.originalRoutes,
      persistent = options.persistent,
      location = options.location,
      history = options.history,
      children = options.children,
      propsActionRef = options.actionRef,
      setTabName = options.setTabName;

  var currentTabLocation = _omit(location, ['key']);

  var cacheName = _get(persistent, 'cacheName', 'tabLocations');

  var actionRef = useRef();

  var _useLocalStorageState = useLocalStorageState(cacheName, []),
      _useLocalStorageState2 = _slicedToArray(_useLocalStorageState, 2),
      tabLocations = _useLocalStorageState2[0],
      setTabLocations = _useLocalStorageState2[1];

  var _useState = useState(function () {
    if (persistent && _isArray(tabLocations) && tabLocations.length) {
      return tabLocations.map(function (tabLocation) {
        var renderRoute = getRenderRoute({
          location: tabLocation,
          mode: mode,
          originalRoutes: originalRoutes,
          setTabName: setTabName
        });
        return {
          title: renderRoute.name,
          key: getRenderRouteKey(renderRoute, mode),
          content: /*#__PURE__*/React.cloneElement(children, {
            location: tabLocation
          }),
          location: tabLocation
        };
      });
    }

    return [];
  }),
      _useState2 = _slicedToArray(_useState, 2),
      tabs = _useState2[0],
      setTabs = _useState2[1];

  var currentRenderRoute = getRenderRoute({
    location: currentTabLocation,
    mode: mode,
    originalRoutes: originalRoutes,
    setTabName: setTabName
  });
  var currentTabKey = useMemo(function () {
    return getRenderRouteKey(currentRenderRoute, mode);
  }, [mode, currentRenderRoute]);
  var prevActiveKey = usePrevious(currentTabKey, function (prev, next) {
    return prev !== next;
  });
  var getTab = usePersistFn(function (tabKey) {
    return _find(tabs, {
      key: tabKey
    });
  });
  var processTabs = usePersistFn(function (_tabs) {
    return _tabs.map(function (item) {
      return _tabs.length === 1 ? _objectSpread(_objectSpread({}, item), {}, {
        closable: false
      }) : item;
    });
  });
  /** 获取激活标签页的相邻标签页 */

  var getNextTab = usePersistFn(function () {
    var removeIndex = _findIndex(tabs, {
      key: currentTabKey
    });

    var nextIndex = removeIndex >= 1 ? removeIndex - 1 : removeIndex + 1;
    return tabs[nextIndex];
  });
  /**
   * force: 是否在目标标签页不存在的时候强制回调函数
   */

  var handleSwitch = usePersistFn(function (keyToSwitch, callback) {
    var force = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : false;

    if (!keyToSwitch) {
      return;
    }
    /**
     * `keyToSwitch` 有值时，`targetTab` 可能为空。
     *
     * 如：一个会调用 `window.closeAndGoBackTab(path)` 的页面在 F5 刷新之后
     */


    var targetTab = getTab(keyToSwitch);

    if (targetTab) {
      history.push(targetTab.location);
    } else {
      history.push(keyToSwitch);
    }

    if (force) {
      callback === null || callback === void 0 ? void 0 : callback();
    } else {
      targetTab && (callback === null || callback === void 0 ? void 0 : callback());
    }
  });
  /** 删除标签页处理事件，可接收一个 `nextTabKey` 参数，自定义需要返回的标签页 */

  var handleRemove = usePersistFn(function (removeKey, nextTabKey, callback, force) {
    if (tabs.length === 1) {
      console.warn('the final tab, can not remove.');
      return;
    }

    var getNextTabKeyByRemove = function getNextTabKeyByRemove() {
      var _getNextTab;

      return removeKey === currentTabKey ? (_getNextTab = getNextTab()) === null || _getNextTab === void 0 ? void 0 : _getNextTab.key : currentTabKey;
    };

    handleSwitch(nextTabKey || getNextTabKeyByRemove(), callback, force);
    setTabs(function (prevTabs) {
      return processTabs(prevTabs.filter(function (item) {
        return item.key !== removeKey;
      }));
    });
  });
  var handleRemoveOthers = usePersistFn(function (currentKey, callback) {
    handleSwitch(currentKey, callback);
    setTabs(function (prevTabs) {
      return processTabs(prevTabs.filter(function (item) {
        return item.key === currentKey;
      }));
    });
  });
  var handRemoveRightTabs = usePersistFn(function (currentKey, callback) {
    handleSwitch(getTab(currentKey).key, callback);
    setTabs(function (prevTabs) {
      return processTabs(prevTabs.slice(0, _findIndex(prevTabs, {
        key: currentKey
      }) + 1));
    });
  });
  /**
   * 新增第一个 tab 不可删除
   *
   * @param newTab
   */

  var addTab = usePersistFn(function (newTab, follow) {
    setTabs(function (prevTabs) {
      var result = _toConsumableArray(prevTabs);

      if (follow) {
        var targetIndex = _findIndex(prevTabs, function (tab) {
          if (mode === Mode.Route) {
            return tab.key === follow;
          }

          var followReg = new RegExp("^".concat(follow));
          return followReg.test(tab.key);
        });

        if (targetIndex >= 0) {
          result.splice(targetIndex + 1, 0, newTab);
        } else {
          result = [].concat(_toConsumableArray(result), [newTab]);
        }
      } else {
        result = [].concat(_toConsumableArray(result), [newTab]);
      }

      return result.map(function (item, index) {
        return tabs.length === 0 && index === 0 ? _objectSpread(_objectSpread({}, item), {}, {
          closable: false
        }) : _objectSpread(_objectSpread({}, item), {}, {
          closable: true
        });
      });
    });
  });
  /**
   * 重载标签页，传入参数重写相关属性
   *
   * @param reloadKey 需要刷新的 tab key
   * @param tabTitle 需要刷新的 tab 标题
   * @param location 需要刷新的 tab location
   * @param content 需要刷新的 tab 渲染的内容
   */

  var reloadTab = usePersistFn(function () {
    var reloadKey = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : currentTabKey;
    var tabTitle = arguments.length > 1 ? arguments[1] : undefined;
    var tabLocation = arguments.length > 2 ? arguments[2] : undefined;
    var content = arguments.length > 3 ? arguments[3] : undefined;

    if (tabs.length < 1) {
      return;
    }

    if (process.env.NODE_ENV === 'development') {
      console.log("reload tab key: ".concat(reloadKey));
    }

    var updatedTabs = tabs.map(function (item) {
      if (item.key === reloadKey) {
        var prevTabTitle = item.title,
            prevLocation = item.location,
            prevContent = item.content,
            rest = _objectWithoutProperties(item, _excluded);

        return _objectSpread(_objectSpread({}, rest), {}, {
          title: tabTitle || prevTabTitle,
          location: tabLocation || prevLocation,
          content: content || /*#__PURE__*/React.cloneElement(item.content, {
            key: new Date().valueOf()
          })
        });
      }

      return item;
    });
    setTabs(updatedTabs);
  });
  var goBackTab = usePersistFn(function (path, callback, force) {
    if (!path && (!prevActiveKey || !getTab(prevActiveKey))) {
      console.warn('go back failed, no previous activated key or previous tab is closed.');
      return;
    }

    handleSwitch(path || prevActiveKey, callback, force);
  });
  /** 关闭后自动切换到附近的标签页，如果是最后一个标签页不可删除 */

  var closeTab = usePersistFn(function (path, callback, force) {
    if (path && !getTab(path)) {
      console.warn('close failed, target tab is closed.');
      return;
    }

    handleRemove(path || currentTabKey, undefined, callback, force);
  });
  /** 关闭当前标签页并返回到上次打开的标签页 */

  var closeAndGoBackTab = usePersistFn(function (path, callback, force) {
    if (!path && (!prevActiveKey || !getTab(prevActiveKey))) {
      console.warn('close and go back failed, no previous activated key or previous tab is closed.');
      return;
    }

    handleRemove(currentTabKey, path || prevActiveKey, callback, force);
  });
  useEffect(function () {
    if (persistent) {
      setTabLocations(tabs.map(function (item) {
        return item.location;
      }));
      return;
    }

    if (tabLocations) {
      setTabLocations();
    }
  }, [persistent, tabs]);
  useEffect(function () {
    actionRef.current = {
      reloadTab: reloadTab,
      goBackTab: goBackTab,
      closeTab: closeTab,
      closeAndGoBackTab: closeAndGoBackTab
    };
    return function () {
      var hint = function hint() {
        console.warn("useSwitchTabs had unmounted.");
      };

      actionRef.current = {
        reloadTab: hint,
        goBackTab: hint,
        closeTab: hint,
        closeAndGoBackTab: hint
      };
    };
  }, []);
  useEffect(function () {
    var activatedTab = getTab(currentTabKey);

    if (activatedTab) {
      var prevTabLocation = activatedTab.location;

      if (!_isEqual(currentTabLocation, prevTabLocation)) {
        reloadTab(currentTabKey, currentRenderRoute.name, currentTabLocation, children);
      } else {
        if (process.env.NODE_ENV === 'development') {
          console.log("no effect of tab key: ".concat(currentTabKey));
        }
      }
    } else {
      var newTab = {
        title: currentRenderRoute.name,
        key: currentTabKey,
        content: children,
        location: currentTabLocation
      };

      var _ref = currentRenderRoute || {},
          follow = _ref.follow;

      if (process.env.NODE_ENV === 'development') {
        console.log("add tab key: ".concat(currentTabKey));
      }

      addTab(newTab, follow);
    } // 不可将当前 location 作为依赖，否则在操作非当前 location 对应的标签页时会有异常
    // 比如在非当前 location 对应的标签页的标签菜单中触发删除其他标签页，会导致本应只有一个标签页时，
    // 但会再次创建一个当前 location 对应的标签页

  }, [children, currentTabKey]);

  if (propsActionRef) {
    if (typeof propsActionRef === 'function') {
      propsActionRef(actionRef.current);
    } else {
      propsActionRef.current = actionRef.current;
    }
  }

  return {
    tabs: tabs,
    activeKey: currentTabKey,
    handleSwitch: handleSwitch,
    handleRemove: handleRemove,
    handleRemoveOthers: handleRemoveOthers,
    handRemoveRightTabs: handRemoveRightTabs
  };
}

export default useSwitchTabs;
